import { onMounted, onUnmounted, ref, isRef, computed } from '@vue/composition-api';

function getQuery(min, max) {
    if (min === void 0) { min = 0; }
    if (max === void 0) { max = 0; }
    if (!min && !max) {
        throw new Error("Please specify at least one value, either 'min' or 'max'");
    }
    else if (min && !max) {
        return "only screen and (min-width: " + min + "px)";
    }
    else if (!min && max) {
        return "only screen and (max-width: " + (max - 1) + "px)";
    }
    else {
        return "only screen and (min-width: " + min + "px) and (max-width: " + (max - 1) + "px)";
    }
}

var defaultEvents = ['mousedown', 'touchstart'];
function useClickAway(elRef, callback, events) {
    if (events === void 0) { events = defaultEvents; }
    var handler = function (e) {
        if (elRef.value && !elRef.value.contains(e.target)) {
            callback(e);
        }
    };
    onMounted(function () {
        events.forEach(function (evtName) { return document.addEventListener(evtName, handler); });
    });
    onUnmounted(function () {
        events.forEach(function (evtName) { return document.removeEventListener(evtName, handler); });
    });
}

function useFullscreen(elRef) {
    var isFullscreen = ref(false);
    var handleFullscreenChange = function () {
        if (!isFullscreen.value)
            return;
        var isCurrentlyFullscreen = !!document.fullscreenElement;
        isFullscreen.value = isCurrentlyFullscreen;
    };
    var startTracking = function () {
        document.addEventListener('fullscreenchange', handleFullscreenChange);
    };
    var stopTracking = function () {
        document.removeEventListener('fullscreenchange', handleFullscreenChange);
    };
    var start = function () {
        if (!elRef.value)
            return;
        if (isFullscreen.value)
            return;
        elRef.value.requestFullscreen().then(function () {
            isFullscreen.value = true;
            startTracking();
        });
    };
    var stop = function () {
        if (!isFullscreen.value)
            return;
        document.exitFullscreen().then(function () {
            isFullscreen.value = false;
            stopTracking();
        });
    };
    onUnmounted(stopTracking);
    return { isFullscreen: isFullscreen, start: start, stop: stop };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var defaultOpts = {
    enableHighAccuracy: false,
    timeout: Infinity,
    maximumAge: 0
};
function useGeolocation(options, runOnMount) {
    if (options === void 0) { options = {}; }
    if (runOnMount === void 0) { runOnMount = true; }
    options = Object.assign(defaultOpts, options);
    // Note: surprisingly the watchId can be 0 (not positive) so
    // we have to check if watchId !== null every time
    var watchId = null;
    var geoInitData = {
        loading: false,
        accuracy: null,
        altitude: null,
        altitudeAccuracy: null,
        heading: null,
        latitude: null,
        longitude: null,
        speed: null,
        timestamp: null
    };
    var isTracking = ref(false);
    var geo = ref(__assign({}, geoInitData));
    var onEventError = function (error) {
        if (watchId === null)
            return;
        geo.value.loading = false;
        geo.value.error = {
            code: error.code,
            message: error.message
        };
        isTracking.value = false;
    };
    var handleGeolocation = function (_a) {
        var coords = _a.coords, timestamp = _a.timestamp;
        geo.value = {
            loading: false,
            accuracy: coords.accuracy,
            altitude: coords.altitude,
            altitudeAccuracy: coords.altitudeAccuracy,
            heading: coords.heading,
            latitude: coords.latitude,
            longitude: coords.longitude,
            speed: coords.speed,
            timestamp: timestamp
        };
        isTracking.value = true;
    };
    var start = function () {
        if (watchId !== null)
            return;
        geo.value.loading = true;
        geo.value.timestamp = Date.now();
        navigator.geolocation.getCurrentPosition(handleGeolocation, onEventError, options);
        watchId = navigator.geolocation.watchPosition(handleGeolocation, onEventError, options);
    };
    var stop = function () {
        if (watchId === null)
            return;
        navigator.geolocation.clearWatch(watchId);
        watchId = null;
        isTracking.value = false;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { isTracking: isTracking, geo: geo, start: start, stop: stop };
}

function useHover(elRef) {
    var isHovered = ref(false);
    var mouseEnterHandler = function () {
        isHovered.value = true;
    };
    var mouseLeaveHandler = function () {
        isHovered.value = false;
    };
    onMounted(function () {
        if (!elRef.value)
            return;
        elRef.value.addEventListener('mouseenter', mouseEnterHandler);
        elRef.value.addEventListener('mouseleave', mouseLeaveHandler);
    });
    onUnmounted(function () {
        if (!elRef.value)
            return;
        elRef.value.removeEventListener('mouseenter', mouseEnterHandler);
        elRef.value.removeEventListener('mouseleave', mouseLeaveHandler);
    });
    return isHovered;
}

/* eslint-disable no-undefined,no-param-reassign,no-shadow */

/**
 * Throttle execution of a function. Especially useful for rate limiting
 * execution of handlers on events like resize and scroll.
 *
 * @param  {Number}    delay          A zero-or-greater delay in milliseconds. For event callbacks, values around 100 or 250 (or even higher) are most useful.
 * @param  {Boolean}   [noTrailing]   Optional, defaults to false. If noTrailing is true, callback will only execute every `delay` milliseconds while the
 *                                    throttled-function is being called. If noTrailing is false or unspecified, callback will be executed one final time
 *                                    after the last throttled-function call. (After the throttled-function has not been called for `delay` milliseconds,
 *                                    the internal counter is reset)
 * @param  {Function}  callback       A function to be executed after delay milliseconds. The `this` context and all arguments are passed through, as-is,
 *                                    to `callback` when the throttled-function is executed.
 * @param  {Boolean}   [debounceMode] If `debounceMode` is true (at begin), schedule `clear` to execute after `delay` ms. If `debounceMode` is false (at end),
 *                                    schedule `callback` to execute after `delay` ms.
 *
 * @return {Function}  A new, throttled, function.
 */
function throttle (delay, noTrailing, callback, debounceMode) {
  /*
   * After wrapper has stopped being called, this timeout ensures that
   * `callback` is executed at the proper times in `throttle` and `end`
   * debounce modes.
   */
  var timeoutID;
  var cancelled = false; // Keep track of the last time `callback` was executed.

  var lastExec = 0; // Function to clear existing timeout

  function clearExistingTimeout() {
    if (timeoutID) {
      clearTimeout(timeoutID);
    }
  } // Function to cancel next exec


  function cancel() {
    clearExistingTimeout();
    cancelled = true;
  } // `noTrailing` defaults to falsy.


  if (typeof noTrailing !== 'boolean') {
    debounceMode = callback;
    callback = noTrailing;
    noTrailing = undefined;
  }
  /*
   * The `wrapper` function encapsulates all of the throttling / debouncing
   * functionality and when executed will limit the rate at which `callback`
   * is executed.
   */


  function wrapper() {
    var self = this;
    var elapsed = Date.now() - lastExec;
    var args = arguments;

    if (cancelled) {
      return;
    } // Execute `callback` and update the `lastExec` timestamp.


    function exec() {
      lastExec = Date.now();
      callback.apply(self, args);
    }
    /*
     * If `debounceMode` is true (at begin) this is used to clear the flag
     * to allow future `callback` executions.
     */


    function clear() {
      timeoutID = undefined;
    }

    if (debounceMode && !timeoutID) {
      /*
       * Since `wrapper` is being called for the first time and
       * `debounceMode` is true (at begin), execute `callback`.
       */
      exec();
    }

    clearExistingTimeout();

    if (debounceMode === undefined && elapsed > delay) {
      /*
       * In throttle mode, if `delay` time has been exceeded, execute
       * `callback`.
       */
      exec();
    } else if (noTrailing !== true) {
      /*
       * In trailing throttle mode, since `delay` time has not been
       * exceeded, schedule `callback` to execute `delay` ms after most
       * recent execution.
       *
       * If `debounceMode` is true (at begin), schedule `clear` to execute
       * after `delay` ms.
       *
       * If `debounceMode` is false (at end), schedule `callback` to
       * execute after `delay` ms.
       */
      timeoutID = setTimeout(debounceMode ? clear : exec, debounceMode === undefined ? delay - elapsed : delay);
    }
  }

  wrapper.cancel = cancel; // Return the wrapper function.

  return wrapper;
}

var idleEventsList = [
    'mousemove',
    'mousedown',
    'resize',
    'keydown',
    'touchstart',
    'wheel'
];
var oneMinute = 60e3;
function useIdle(ms, events, runOnMount) {
    if (ms === void 0) { ms = oneMinute; }
    if (events === void 0) { events = idleEventsList; }
    if (runOnMount === void 0) { runOnMount = true; }
    var timeout = null;
    var isIdle = ref(false);
    var isTracking = ref(false);
    var handleChange = throttle(50, function () {
        isIdle.value = false;
        if (timeout)
            clearTimeout(timeout);
        timeout = setTimeout(function () {
            isIdle.value = true;
        }, ms);
    });
    var handleVisibility = function () {
        if (!document.hidden)
            return;
        handleChange();
    };
    var start = function () {
        if (isTracking.value)
            return;
        events.forEach(function (evtName) { return document.addEventListener(evtName, handleChange); });
        document.addEventListener('visibilitychange', handleVisibility);
        // Initialize it since the events above may not run immediately
        handleChange();
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        events.forEach(function (evtName) {
            return document.removeEventListener(evtName, handleChange);
        });
        document.removeEventListener('visibilitychange', handleVisibility);
        // Stop timer if it is still running
        if (timeout)
            clearTimeout(timeout);
        // Restore initial status
        timeout = null;
        isIdle.value = false;
        isTracking.value = false;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { isIdle: isIdle, isTracking: isTracking, start: start, stop: stop };
}

var errorMsg = 'IntersectionObserver is not supported, please install a polyfill';
function useIntersection(elRef, options, runOnMount) {
    if (options === void 0) { options = {}; }
    if (runOnMount === void 0) { runOnMount = true; }
    var observedEntry = ref(null);
    var handleObserver = function (entries) {
        observedEntry.value = entries[0];
    };
    var observer = null;
    var start = function () {
        if (!('IntersectionObserver' in window))
            throw new Error(errorMsg);
        // Do not start if the observer is already initialized
        // or the elRef does not exist
        if (observer || !elRef.value)
            return;
        observer = new IntersectionObserver(handleObserver, options);
        observer.observe(elRef.value);
    };
    var stop = function () {
        if (!observer)
            return;
        observer.disconnect();
        observer = null;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { observedEntry: observedEntry, start: start, stop: stop };
}

// The history methods 'pushState' and 'replaceState' by default do not fire an event
// unless it is coming from user interaction with the browser navigation bar,
// so we are adding a patch to make them detectable
var isPatched = false;
var patchHistoryMethodsOnce = function () {
    if (isPatched)
        return;
    var methods = ['pushState', 'replaceState'];
    methods.forEach(function (method) {
        var original = history[method];
        history[method] = function (state) {
            // eslint-disable-next-line prefer-rest-params
            var result = original.apply(this, arguments);
            var event = new Event(method.toLowerCase());
            event.state = state;
            window.dispatchEvent(event);
            return result;
        };
    });
    isPatched = true;
};
var normalizeEntriesData = function (data) {
    return data.reduce(function (acc, _a) {
        var key = _a[0], val = _a[1];
        acc[key] = val;
        return acc;
    }, {});
};

function useLocation(runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var buildState = function (trigger) {
        var state = history.state, length = history.length;
        var hash = location.hash, host = location.host, hostname = location.hostname, href = location.href, origin = location.origin, pathname = location.pathname, port = location.port, protocol = location.protocol, search = location.search;
        return {
            trigger: trigger,
            state: state,
            length: length,
            hash: hash,
            host: host,
            hostname: hostname,
            href: href,
            origin: origin,
            pathname: pathname,
            port: port,
            protocol: protocol,
            search: search
        };
    };
    var isTracking = ref(false);
    var locationState = ref(buildState('load'));
    var popState = function () { return (locationState.value = buildState('popstate')); };
    var pushState = function () { return (locationState.value = buildState('pushstate')); };
    var replaceState = function () { return (locationState.value = buildState('replacestate')); };
    var start = function () {
        if (isTracking.value)
            return;
        patchHistoryMethodsOnce();
        locationState.value = buildState('start');
        window.addEventListener('popstate', popState);
        window.addEventListener('pushstate', pushState);
        window.addEventListener('replacestate', replaceState);
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('popstate', popState);
        window.removeEventListener('pushstate', pushState);
        window.removeEventListener('replacestate', replaceState);
        isTracking.value = false;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { locationState: locationState, isTracking: isTracking, start: start, stop: stop };
}

function useMedia(query, defaultState) {
    if (defaultState === void 0) { defaultState = false; }
    var mql;
    var matches = ref(defaultState);
    var updateMatchValue = function () { return (matches.value = mql.matches); };
    onMounted(function () {
        mql = window.matchMedia(query);
        mql.addListener(updateMatchValue);
        updateMatchValue();
    });
    onUnmounted(function () {
        mql.removeListener(updateMatchValue);
    });
    return matches;
}

function useMouse() {
    var docX = ref(0);
    var docY = ref(0);
    var mouseMoveHandler = function (e) {
        docX.value = e.pageX;
        docY.value = e.pageY;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY
    };
}

function useMouseElement(elRef) {
    var docX = ref(0);
    var docY = ref(0);
    var elX = ref(0);
    var elY = ref(0);
    var elInfoX = ref(0);
    var elInfoY = ref(0);
    var elInfoW = ref(0);
    var elInfoH = ref(0);
    var mouseMoveHandler = function (e) {
        if (!elRef.value)
            return;
        docX.value = e.pageX;
        docY.value = e.pageY;
        var _a = elRef.value.getBoundingClientRect(), left = _a.left, top = _a.top, height = _a.height, width = _a.width;
        elInfoX.value = left + window.pageXOffset;
        elInfoY.value = top + window.pageYOffset;
        elInfoW.value = width;
        elInfoH.value = height;
        elX.value = e.pageX - elInfoX.value;
        elY.value = e.pageY - elInfoY.value;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY,
        elX: elX,
        elY: elY,
        elInfoX: elInfoX,
        elInfoY: elInfoY,
        elInfoW: elInfoW,
        elInfoH: elInfoH
    };
}

var normalizeParams = function (urlParamsObj) { return function (paramAcc, param) {
    paramAcc[param] = param in urlParamsObj ? urlParamsObj[param] : null;
    return paramAcc;
}; };
var getUrlParams = function (parameters) {
    var urlParamsObj = normalizeEntriesData(Array.from(new URLSearchParams(location.search).entries()));
    return parameters.reduce(normalizeParams(urlParamsObj), {});
};
function useSearchParams(parameters, runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var searchParams = ref(parameters.reduce(normalizeParams({}), {}));
    var isTracking = ref(false);
    var handleParamsChange = function () {
        return (searchParams.value = getUrlParams(parameters));
    };
    var start = function () {
        if (isTracking.value)
            return;
        patchHistoryMethodsOnce();
        handleParamsChange();
        window.addEventListener('popstate', handleParamsChange);
        window.addEventListener('pushstate', handleParamsChange);
        window.addEventListener('replacestate', handleParamsChange);
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('popstate', handleParamsChange);
        window.removeEventListener('pushstate', handleParamsChange);
        window.removeEventListener('replacestate', handleParamsChange);
        isTracking.value = false;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { searchParams: searchParams, isTracking: isTracking, start: start, stop: stop };
}

function useIntervalFn(callback, ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isRunning = ref(false);
    var interval = null;
    var start = function () {
        if (interval)
            return;
        isRunning.value = true;
        interval = setInterval(callback, ms);
    };
    var stop = function () {
        clearInterval(interval);
        isRunning.value = false;
        interval = null;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { isRunning: isRunning, start: start, stop: stop };
}

function useInterval(ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var counter = ref(0);
    var animHandler = function () {
        counter.value = counter.value + 1;
    };
    var _a = useIntervalFn(animHandler, ms, runOnMount), isRunning = _a.isRunning, start = _a.start, stop = _a.stop;
    return { isRunning: isRunning, counter: counter, start: start, stop: stop };
}

var getFps = function (fps) { return Number(isRef(fps) ? fps.value : fps); };
var calcFpsInterval = function (fps) { return 1000 / fps; };
var fpsLimit = 60;
function useRafFn(callback, 
// Note: a value equal to or greater than 60 will disable the fps check logic
// giving maximum precision and smoothness
fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isRunning = ref(false);
    // Using a computed here allows us to update fps
    // dynamically from user's input
    var fpsInterval = computed(function () { return calcFpsInterval(getFps(fps)); });
    var itWasIdle = false;
    var startTime = 0;
    var timeElapsed = 0;
    var timePausedTot = 0;
    var timeDelta = 0;
    var loop = function (timeStamp) {
        if (!startTime)
            startTime = timeStamp;
        if (!isRunning.value)
            return;
        // Store the time that the loop is idle so that we can
        // calculate the timeElapsed variable correctly and return
        // it to the user from within the callback
        if (itWasIdle) {
            timePausedTot = timeStamp - startTime - timeElapsed;
            itWasIdle = false;
        }
        // Adjust timeElapsed to account for startTime and timePausedTot
        timeElapsed = timeStamp - startTime - timePausedTot;
        // Always run the callback if fps is greater than fpsLimit
        var callbackShouldAlwaysRun = getFps(fps) >= fpsLimit;
        if (callbackShouldAlwaysRun) {
            // Store timeDelta for future computations
            timeDelta = timeElapsed;
            callback(timeElapsed);
        }
        // Run callback only when !callbackShouldAlwaysRun
        // and the given fps matches the lapsed time
        if (!callbackShouldAlwaysRun) {
            var elapsedTimeFromPrevLoop = Math.ceil(timeElapsed - timeDelta);
            if (elapsedTimeFromPrevLoop > fpsInterval.value) {
                // Store timeDelta for future computations
                timeDelta = timeElapsed;
                callback(timeElapsed);
            }
        }
        // Run loop again recursively
        requestAnimationFrame(loop);
    };
    var start = function () {
        isRunning.value = true;
        requestAnimationFrame(loop);
    };
    var stop = function () {
        isRunning.value = false;
        itWasIdle = true;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return {
        isRunning: isRunning,
        start: start,
        stop: stop
    };
}

var fpsLimit$1 = 60;
function useRaf(fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit$1; }
    if (runOnMount === void 0) { runOnMount = true; }
    var elapsed = ref(0);
    var animHandler = function (elapsedTime) {
        elapsed.value = elapsedTime;
    };
    var _a = useRafFn(animHandler, fps, runOnMount), isRunning = _a.isRunning, start = _a.start, stop = _a.stop;
    return {
        isRunning: isRunning,
        elapsed: elapsed,
        start: start,
        stop: stop
    };
}

function useTimeoutFn(callback, ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isReady = ref(null);
    var timeout = null;
    var start = function () {
        isReady.value = false;
        if (timeout)
            clearTimeout(timeout);
        timeout = setTimeout(function () {
            isReady.value = true;
            callback();
            timeout = null;
        }, ms);
    };
    var stop = function () {
        isReady.value = null;
        if (timeout) {
            clearTimeout(timeout);
            timeout = null;
        }
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return { isReady: isReady, start: start, stop: stop };
}

var noop = function () { return null; };
function useTimeout(ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var _a = useTimeoutFn(noop, ms, runOnMount), isReady = _a.isReady, start = _a.start, stop = _a.stop;
    return { isReady: isReady, start: start, stop: stop };
}

export { getQuery, useClickAway, useFullscreen, useGeolocation, useHover, idleEventsList, useIdle, useIntersection, useLocation, useMedia, useMouse, useMouseElement, useSearchParams, useIntervalFn, useInterval, useRafFn, useRaf, useTimeoutFn, useTimeout };
